#include "DeckListView.h"

#include "pixmaps.h"

#include <qdragobject.h>
#include <qpopupmenu.h>

#include <assert.h>
#include <iostream>

namespace
{

    using namespace pitacard::ui;

    const char deckMimeType[] = "DeckListViewItem";

    QListViewItem* encodedToPointer(const QMimeSource& ms, const char* type)
    {
	QByteArray ba = ms.encodedData(type);
	QListViewItem* rval = NULL;
	memcpy(&rval, ba.data(), sizeof(rval));

	return rval;
    }
    
    class DeckMenu : public QPopupMenu
    {
    public:
	enum
	{
	    ADD_CARD,
	    REMOVE_CARD,
	    ADD_FOLDER,
	    REMOVE_FOLDER
	};
	
	DeckMenu(QWidget* parent=0,
		 const char* name=0) :
	    QPopupMenu (parent, name)
	    {
		insertItem("Add card", ADD_CARD);
		insertItem("Remove card", REMOVE_CARD);
		insertItem("Add folder", ADD_FOLDER);
		insertItem("Remove folder", REMOVE_FOLDER);
	    }
    };

    template <class T>
    struct GenericDragObject : public QStoredDrag
    {
	GenericDragObject(T item,
			  const QString& mimeType,
			  QWidget* dragSource=0,
			  const char* name=0) :
	    QStoredDrag (mimeType, dragSource, name),
	    mItem (item)
	    {
		setPixmap(pitacard::pixmaps::folderClosed());
	    }

	QByteArray encodedData(const char* name) const
	    {
		// TODO: Switch on name
		
		char buf[sizeof(mItem)];
		memcpy(buf, &mItem, sizeof(mItem));
		QByteArray b;
		b.duplicate(buf, sizeof(mItem));
		return b;
	    }

	T mItem;
    };

    bool isChildOf(const QListViewItem* child,
		   const QListViewItem* parent)
    {
	if (!parent || !child) return false;
	
	for (QListViewItem* p = child->parent();
	     p != NULL;
	     p = p->parent())
	{
	    if (p == parent) return true;
	}

	return false;
    }

    template <class Parent>
    void reparent(QListViewItem* child, Parent* p)
    {
	if (!child || !p) return;

	if (child->parent())
	    child->parent()->takeItem(child);
	else if (child->listView())
	    child->listView()->takeItem(child);

	p->insertItem(child);
    }
    
}

namespace pitacard { namespace ui {

    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    CardListViewItem::CardListViewItem(const Card& card,
				       QListView* parent) :
	QListViewItem(parent),
	mCard(card)
    {
	setDragEnabled(true);
	setDropEnabled(true);
	setRenameEnabled(0, true);
	setText(0, mCard.name());
    }
    
    CardListViewItem::CardListViewItem(const Card& card,
				       QListViewItem* parent) :
	QListViewItem (parent),
	mCard(card)
    {
	setDragEnabled(true);
	setDropEnabled(true);
	setRenameEnabled(0, true);
	setText(0, mCard.name());
    }
    
    CardListViewItem::CardListViewItem(const Card& card,
				       QListView* parent, QListViewItem* after) :
	QListViewItem(parent, after),
	mCard(card)
    {
	setDragEnabled(true);
	setDropEnabled(true);
	setRenameEnabled(0, true);
	setText(0, mCard.name());
    }
    
    CardListViewItem::CardListViewItem(const Card& card,
				       QListViewItem* parent, QListViewItem* after) :
	QListViewItem(parent, after),
	mCard(card)
    {
	setDragEnabled(true);
	setDropEnabled(true);
	setRenameEnabled(0, true);
	setText(0, mCard.name());
    }
    
    bool CardListViewItem::acceptDrop(const QMimeSource* src) const
    {
	if (!src->provides(deckMimeType)) return false;
	QListViewItem* dropped = encodedToPointer(*src, deckMimeType);
	if (!dropped) return false;
	if (isChildOf(this, dropped)) return false;
	return true;
    }

#if 0
    void CardListViewItem::dropped(QDropEvent* e)
    {
	QListViewItem* dropped = encodedToPointer(*e, deckMimeType);
	assert(dropped);
	assert(isChildOf(this, dropped) == false);
	
	if (parent())
	    ::reparent(dropped, parent());
	else
	    ::reparent(dropped, listView());
    }
#endif

    const Card& CardListViewItem::card() const
    {
	return mCard;
    }

    void CardListViewItem::setCard(const Card& card)
    {
	mCard = card;
	setText(0, mCard.name());
    }


    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    FolderListViewItem::FolderListViewItem(const QString& name,
					   QListView* parent) :
	QListViewItem(parent),
	mName(name)
    {
	setDragEnabled(true);
	setDropEnabled(true);
	setRenameEnabled(0, true);
	setPixmap(0, pitacard::pixmaps::folderOpen());
	setText(0, mName);
    }
    
    FolderListViewItem::FolderListViewItem(const QString& name,
					   QListViewItem* parent) :
	QListViewItem (parent),
	mName(name)
    {
	setDragEnabled(true);
	setDropEnabled(true);
	setRenameEnabled(0, true);
	setPixmap(0, pitacard::pixmaps::folderOpen());
	setText(0, mName);
    }
    
    FolderListViewItem::FolderListViewItem(const QString& name,
					   QListView* parent, QListViewItem* after) :
	QListViewItem(parent, after),
	mName(name)
    {
	setDragEnabled(true);
	setDropEnabled(true);
	setRenameEnabled(0, true);
	setPixmap(0, pitacard::pixmaps::folderOpen());
	setText(0, mName);
    }
    
    FolderListViewItem::FolderListViewItem(const QString& name,
					   QListViewItem* parent, QListViewItem* after) :
	QListViewItem(parent, after),
	mName(name)
    {
	setDragEnabled(true);
	setDropEnabled(true);
	setRenameEnabled(0, true);
	setPixmap(0, pitacard::pixmaps::folderOpen());
	setText(0, mName);
    }

    bool FolderListViewItem::acceptDrop(const QMimeSource* src) const
    {
	if (!src->provides(deckMimeType)) return false;
	QListViewItem* dropped = encodedToPointer(*src, deckMimeType);
	if (!dropped) return false;
	if (isChildOf(this, dropped)) return false;
	return true;
    }

#if 0
    void FolderListViewItem::dropped(QDropEvent* e)
    {
	QListViewItem* dropped = encodedToPointer(*e, deckMimeType);
	assert(dropped);
	assert(isChildOf(this, dropped) == false);
	::reparent(dropped, this);
    }
#endif

    const QString& FolderListViewItem::name() const
    {
	return mName;
    }

    void FolderListViewItem::setName(const QString& name)
    {
	mName = name;
	setText(0, mName);
    }


    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    DeckListView::DeckListView(QWidget* parent,
			       const char* name,
			       WFlags f) :
	QListView (parent, name, f)
    {
	setSorting(columns() + 1);

	setAcceptDrops(true);
	
	addColumn("Card Name");
	setRootIsDecorated(true);

	setDefaultRenameAction(Accept);

	connect(this,
		SIGNAL(rightButtonPressed( QListViewItem *, const QPoint &, int )),
		SLOT(handleContextMenu( QListViewItem *, const QPoint &, int )));

	connect(this,
		SIGNAL(itemRenamed(QListViewItem*, int, const QString&)),
		SLOT(renameItem(QListViewItem*, int, const QString&)));

	connect(this,
		SIGNAL(doubleClicked(QListViewItem*)),
		SLOT(initiateRename(QListViewItem*)));
    }

    void DeckListView::addCard()
    {
	CardListViewItem* item = NULL;
	if (currFolder())
	    item = new CardListViewItem(Card(), currFolder());
	else if (currCard() && currCard()->parent())
	    item = new CardListViewItem(Card(), currCard()->parent());
	else
	    item = new CardListViewItem(Card(), this);

	assert(item);
	setSelected(item, true);

	item->startRename(0);
	
	emit dataChanged();
    }
    
    void DeckListView::removeCard()
    {
	if (currCard())
	{
	    // delete card
	    delete currCard();

	    emit dataChanged();
	}
    }
    
    void DeckListView::addFolder()
    {
	FolderListViewItem* flvi = NULL;
	if (currFolder())
	    flvi = new FolderListViewItem("", currFolder());
	else if (currCard() && currCard()->parent())
	    flvi = new FolderListViewItem("", currCard()->parent());
	else
	    flvi = new FolderListViewItem("", this);

	assert(flvi);
	setSelected(flvi, true);

	flvi->startRename(0);
	
	emit dataChanged();
    }
    
    void DeckListView::removeFolder()
    {
	if (currFolder())
	{
	    // delete card
	    delete currFolder();

	    emit dataChanged();
	}
    }

    CardListViewItem* DeckListView::currCard()
    {
	return dynamic_cast<CardListViewItem*>(selectedItem());
    }
    
    FolderListViewItem* DeckListView::currFolder()
    {
	return dynamic_cast<FolderListViewItem*>(selectedItem());
    }
    
    QDragObject* DeckListView::dragObject()
    {
	if (QListViewItem* lvi = selectedItem())
	    return new GenericDragObject<QListViewItem*>(lvi, deckMimeType, this);
	else
	    return NULL;
    }

    void DeckListView::dragMoveEvent(QDragMoveEvent* evt)
    {
	QListViewItem* item = itemAt(evt->pos());
	if (item)
	    evt->accept(item->acceptDrop(evt));
	else
	    evt->accept();
    }

    void DeckListView::dropEvent(QDropEvent* evt)
    {
	QListViewItem* dropped = encodedToPointer(*evt, deckMimeType);
	if (!dropped) return;
	    
	QListViewItem* newParent = itemAt(evt->pos());

	if (!newParent)
	{
	    ::reparent(dropped, this);
	    emit dataChanged();
	}
	else if (isChildOf(newParent, dropped))
	    return;
	else if (FolderListViewItem* flvi =
		 dynamic_cast<FolderListViewItem*>(newParent))
	{
	    ::reparent(dropped, flvi);
	    emit dataChanged();
	}
	else if (dynamic_cast<CardListViewItem*>(newParent))
	{
	    if (newParent->parent())
		::reparent(dropped, newParent->parent());
	    else
		::reparent(dropped, newParent->listView());

	    emit dataChanged();
	}
    }
    
    void DeckListView::handleContextMenu(QListViewItem* item,
					 const QPoint& pt,
					 int idx)
    {
	if (isRenaming()) return;
	
	DeckMenu dcm;
	switch (dcm.exec(pt))
	{
	    case DeckMenu::ADD_CARD:
		addCard();
		break;
		
	    case DeckMenu::REMOVE_CARD:
		removeCard();
		break;

	    case DeckMenu::ADD_FOLDER:
		addFolder();
		break;


	    case DeckMenu::REMOVE_FOLDER:
		removeFolder();
		break;
		
	    default:
		break;
	}
    }

    void DeckListView::initiateRename(QListViewItem* item)
    {
	if (item) item->startRename(0);
    }
    
    void DeckListView::renameItem(QListViewItem* item, int col, const QString& txt)
    {
	if (!item) return;

	if (CardListViewItem* clvi = dynamic_cast<CardListViewItem*>(item))
	{
	    Card c = clvi->card();
	    c.setName(txt);
	    clvi->setCard(c);
	    emit dataChanged();
	}
	else if (FolderListViewItem* flvi = dynamic_cast<FolderListViewItem*>(item))
	{
	    flvi->setName(txt);
	    emit dataChanged();
	}
    }
    
} }
