#ifndef DECKLISTVIEW_H
#define DECKLISTVIEW_H

#include "card.h"

#include <qlistview.h>

class QPoint;

namespace pitacard { namespace ui {

    using namespace pitacard::data;
    
    struct CardListViewItem : public QListViewItem
    {
	CardListViewItem(const Card& card,
			 QListView* parent);
	CardListViewItem(const Card& card,
			 QListViewItem* parent);
	CardListViewItem(const Card& card,
			 QListView* parent, QListViewItem* after);
	CardListViewItem(const Card& card,
			 QListViewItem* parent, QListViewItem* after);

	bool acceptDrop(const QMimeSource* src) const;

	const Card& card() const;
	void setCard(const Card& card);
	
    private:
	Card mCard;
    };

    struct FolderListViewItem : public QListViewItem
    {
	FolderListViewItem(const QString& name,
			   QListView* parent);
	FolderListViewItem(const QString& name,
			   QListViewItem* parent);
	FolderListViewItem(const QString& name,
			   QListView* parent, QListViewItem* after);
	FolderListViewItem(const QString& name,
			   QListViewItem* parent, QListViewItem* after);

	bool acceptDrop(const QMimeSource* src) const;
	
	const QString& name() const;
	void setName(const QString& name);
	
    private:
	QString mName;
    };
    
    class DeckListView : public QListView
    {
	Q_OBJECT
	
    public:
	DeckListView(QWidget* parent=0,
		     const char* name=0,
		     WFlags f=0);

	void addCard();
	void removeCard();
	void addFolder();
	void removeFolder();

	CardListViewItem* currCard();
	FolderListViewItem* currFolder();

    signals:
	void dataChanged();
	
    protected:
	QDragObject* dragObject();
	void dragMoveEvent(QDragMoveEvent* evt);
	void dropEvent(QDropEvent* evt);
	
    protected slots:
        void initiateRename(QListViewItem* item);
	void renameItem(QListViewItem*, int, const QString&);
	void handleContextMenu(QListViewItem*,
			       const QPoint&,
			       int);
    };
    
} }

#endif
