#ifndef DECKRUNNERUI_H
#define DECKRUNNERUI_H

#include "DeckRunnerUIBase.h"
#include "folder.h"
#include "globals.h"
#include "xlatTextEdit.h"

namespace pitacard { namespace ui {

    using namespace pitacard::data;
    using namespace pitacard::config;

    template <class Selector>
    class DeckRunnerUI : public DeckRunnerUIBase
    {
    public:
	typedef vector<Folder::Cards::const_iterator> Cards;
	
	DeckRunnerUI(const Folder& folder,
		     QWidget* parent=0,
		     const char* name=0,
		     bool modal=FALSE,
		     WFlags fl=0) :
	    DeckRunnerUIBase (parent, name, modal, fl)
	    {
		using namespace pitacard::config;

		// construct mCards first
		gatherCards(folder, mCards);
		mCurr = mCards.begin();
		
		mFrontTextEdit->setFont(cardFont());
		mBackTextEdit->setFont(cardFont());
	    }

    protected:
	void nextCard()
	    {
		mCurr = mSelector.nextCard(mCards, mCurr);
	    }

	void showFront()
	    {
		if (mCurr == mCards.end()) return;
		
		mFrontTextEdit->setText(mCurr->front().mText);
	    }

	void hideFront()
	    {
		mFrontTextEdit->setText("");
	    }

	void showBack()
	    {
		if (mCurr == mCards.end()) return;
		
		mBackTextEdit->setText(mCurr->back().mText);
	    }

	void hideBack()
	    {
		mBackTextEdit->setText("");
	    }

	bool haveValidCard() const
	    { return mCurr != mCards.end(); }

	Cards::const_iterator currentCard() const
	    { return mCurr; }

    private:
	static void gatherCards(const Folder& folder,
				DeckRunnerUI<Selector>::Cards& cards)
	    {
		for (Folder::Cards::const_iterator itr = folder.mCards.begin();
		     itr != folder.mCards.end();
		     ++itr)
		{
		    cards.push_back(itr);
		}
		
		for (Folder::Folders::const_iterator itr = folder.mFolders.begin();
		     itr != folder.mFolders.end();
		     ++itr)
		{
		    gatherCards(*itr, cards);
		}
	    }
	
    private:
	Cards mCards;
	Cards::const_iterator mCurr;
	Selector mSelector;
    };
    
} }

#endif
