#include "deckreader.h"

#include "card.h"
#include "folder.h"

#include <iostream>

#include <qdatastream.h>
#include <qfile.h>
#include <qstring.h>

using namespace pitacard::data;

namespace
{

    void readDeck_v1(QDataStream& ds, Folder& deck)
    {
	while (!ds.atEnd())
	{
	    QString name, front, back;
	    ds >> name >> front >> back;
	    deck.mCards.push_back(Card(name, front, back));
	}
    }
    
    void readDeck_v2(QDataStream& ds, Folder& deck)
    {
	QString name;
	ds >> name;

	deck.mName = name;

	Folder::Cards::size_type numCards;
	ds >> numCards;
	for (Folder::Cards::size_type idx = 0;
	     idx < numCards;
	     ++idx)
	{
	    QString cname, ftext, btext;

	    ds >> cname >> ftext >> btext;
	    
	    deck.mCards.push_back(Card(cname, ftext, btext));
	}

	Folder::Folders::size_type numFolders;
	ds >> numFolders;
	deck.mFolders = Folder::Folders(numFolders);
	for (Folder::Folders::size_type idx = 0;
	     idx < numFolders;
	     ++idx)
	{
	    readDeck_v2(ds, deck.mFolders[idx]);
	}
    }
    
}

namespace pitacard { namespace io {

    void readDeck(QDataStream& ds, Folder& deck)
    {
	unsigned int formatVersion;
	ds >> formatVersion;

	switch (formatVersion)
	{
	    case 1:
		readDeck_v1(ds, deck);
		break;

	    case 2:
		readDeck_v2(ds, deck);
		break;
		
	    default:
		std::cerr << "[readDeck] ERROR: unknown format version = "
			  << formatVersion << std::endl;
	}
    }
    
    void readDeck(const QString& filename, Folder& deck)
    {
	QFile file(filename);
	file.open(IO_ReadOnly);
	QDataStream ds(&file);
	readDeck(ds, deck);
    }
    
} }
